@@ -3,7 +3,7 @@
   FILE: icaltime.c
   CREATOR: eric 02 June 2000
   
-  $Id: icaltime.c,v 1.36 2005/03/15 01:48:04 jpr Exp $
+  $Id: icaltime.c,v 1.39 2005/09/05 05:08:10 pchen Exp $
   $Locker:  $
     
  (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org
@@ -51,6 +51,14 @@
 #define strcasecmp    stricmp
 #endif
 
+#ifdef HAVE_PTHREAD
+ #include <pthread.h>    
+    static pthread_mutex_t tzid_mutex = PTHREAD_MUTEX_INITIALIZER;
+#endif
+
+static void release_saved_tz ();
+static char * access_saved_tz ();
+
 /*
  *  Function to convert a struct tm time specification
  *  to an ANSI time_t using the specified time zone.
@@ -201,7 +209,7 @@ icaltime_from_timet_with_zone(const time_t tm, const int is_date,
     tt.zone = NULL;
 
     /* Use our timezone functions to convert to the required timezone. */
-    icaltimezone_convert_time (&tt, utc_zone, zone);
+    icaltimezone_convert_time (&tt, utc_zone, (icaltimezone *)zone);
 
     tt.is_date = is_date; 
 
@@ -317,13 +325,16 @@ char* set_tz(const char* tzid)
     /* Add the new TZ to the environment. */
     putenv(new_tz); 
 
-    /* Free any previous TZ environment string we have used. */
-    if (saved_tz)
+    /* Free any previous TZ environment string we have used in a synchronized manner. */
+
+    if (access_saved_tz ()) 
       free (saved_tz);
 
     /* Save a pointer to the TZ string we just set, so we can free it later. */
     saved_tz = new_tz;
 
+    release_saved_tz ();
+
     return old_tz_copy; /* This will be zero if the TZ env var was not set */
 }
 
@@ -346,13 +357,16 @@ void unset_tz(char *tzstr)
 #endif
     } 
 
-    /* Free any previous TZ environment string we have used. */
-    if (saved_tz)
+    /* Free any previous TZ environment string we have used in a synchronized manner */
+
+    if (access_saved_tz ()) 
 	free (saved_tz);
 
     /* Save a pointer to the TZ string we just set, so we can free it later.
        (This can possibly be NULL if there was no TZ to restore.) */
     saved_tz = tzstr;
+
+    release_saved_tz ();	
 }
 
 /**	Return the time as seconds past the UNIX epoch, using the
@@ -385,7 +399,7 @@ time_t icaltime_as_timet_with_zone(const struct icaltimetype tt,
     local_tt.is_date = 0;
 
     /* Use our timezone functions to convert to UTC. */
-    icaltimezone_convert_time (&local_tt, zone, utc_zone);
+    icaltimezone_convert_time (&local_tt, (icaltimezone *)zone, utc_zone);
 
     /* Copy the icaltimetype to a struct tm. */
     memset (&stm, 0, sizeof (struct tm));
@@ -991,7 +1005,7 @@ struct icaltimetype icaltime_convert_to_zone(const struct icaltimetype tt,
 
 	/* If it's a floating time we don't want to adjust the time */
 	if (tt.zone != NULL) {
-		icaltimezone_convert_time(&ret, tt.zone, zone);
+		icaltimezone_convert_time(&ret, (icaltimezone *)tt.zone, zone);
 	}
 
 	ret.zone = zone;
@@ -1014,7 +1028,7 @@ char *
 icaltime_get_tzid(const struct icaltimetype t) {
 
 	if (t.zone != NULL) {
-		return icaltimezone_get_tzid(t.zone);
+		return icaltimezone_get_tzid((icaltimezone *)t.zone);
 	} else {
 		return NULL;
 	}
@@ -1151,3 +1165,33 @@ int icaltime_span_contains(icaltime_span *s,
   
   return 0;
 }
+
+/* @brief gives access the static variable saved_tz synchronously  
+ * @return		The pointer to the saved_tz
+ *
+ */
+
+static char * access_saved_tz ()
+{
+ 
+	#ifdef HAVE_PTHREAD
+	    pthread_mutex_lock (&tzid_mutex);
+	#endif
+	return saved_tz;
+
+}
+
+
+/* @brief releases the access to the static variable saved_tz synchronously  
+ *
+ */
+
+static void release_saved_tz ()
+{
+ 
+	#ifdef HAVE_PTHREAD
+	    pthread_mutex_unlock (&tzid_mutex);
+	#endif
+	return;
+
+}
