@@ -10,15 +10,15 @@
  * Copyright (C) 2004, Novell Inc.
  *
  * This program is free software; you can redistribute it and/or 
- * modify it under the terms of version 2 of the GNU General Public 
+ * modify it under the terms of version 2 of the GNU Lesser General Public 
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -57,6 +57,7 @@
 #define ADD_JUNK_ENTRY 1
 #define REMOVE_JUNK_ENTRY -1
 #define JUNK_FOLDER "Junk Mail"
+#define READ_CURSOR_MAX_IDS 500
 
 static CamelOfflineFolderClass *parent_class = NULL;
 
@@ -128,6 +129,7 @@ groupwise_folder_get_message( CamelFolder *folder,
 				camel_object_unref (msg);
 				camel_object_unref (cache_stream);
 				camel_object_unref (stream);
+				camel_message_info_free (&mi->info);
 				return NULL;
 			} else {
 				camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot get message %s: %s"),
@@ -141,12 +143,22 @@ groupwise_folder_get_message( CamelFolder *folder,
 	camel_object_unref (stream);
 
 	if (msg != NULL) {
+		camel_message_info_free (&mi->info);
 		return msg;
 	}
 	
 	if (((CamelOfflineStore *) gw_store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
 				_("This message is not available in offline mode."));
+		camel_message_info_free (&mi->info);
+		return NULL;
+	}
+	
+	/* Check if we are really offline */
+	if (!camel_groupwise_store_connected (gw_store, ex)) {
+		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+				_("This message is not available in offline mode."));
+		camel_message_info_free (&mi->info);
 		return NULL;
 	}
 
@@ -157,6 +169,7 @@ groupwise_folder_get_message( CamelFolder *folder,
 	if (status != E_GW_CONNECTION_STATUS_OK) {
 		g_free (container_id);
 		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Could not get message"));
+		camel_message_info_free (&mi->info);
 		return NULL;
 	}
 
@@ -164,10 +177,17 @@ groupwise_folder_get_message( CamelFolder *folder,
 	if (!msg) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Could not get message"));
 		g_free (container_id);
+		camel_message_info_free (&mi->info);
 		
 		return NULL;
 	}
 
+	if (msg)
+		camel_medium_set_header (CAMEL_MEDIUM (msg), "X-Evolution-Source", groupwise_base_url_lookup (priv));
+
+	if(!strcmp (folder->full_name, "Sent Items")) 
+		goto end; /*Dont cache if its sent items, since we need to Track Status*/
+
 	/* add to cache */
 	CAMEL_GROUPWISE_FOLDER_LOCK (folder, cache_lock);
 	if ((cache_stream = camel_data_cache_add (gw_folder->cache, "cache", uid, NULL))) {
@@ -179,9 +199,10 @@ groupwise_folder_get_message( CamelFolder *folder,
 
 	CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
 	
+end:
+	camel_message_info_free (&mi->info);
 	g_free (container_id);
 	return msg;
-
 }
 
 static void
@@ -193,7 +214,7 @@ groupwise_populate_details_from_item (CamelMimeMessage *msg, EGwItem *item)
 	temp_str = (char *)e_gw_item_get_subject(item);
 	if(temp_str)
 		camel_mime_message_set_subject (msg, temp_str);
-	dtstring = e_gw_item_get_creation_date (item);
+	dtstring = e_gw_item_get_delivered_date (item);
 	if(dtstring) {
 		int offset = 0;
 		time_t time = e_gw_connection_get_date_from_string (dtstring);
@@ -462,7 +483,7 @@ update_junk_list (CamelStore *store, CamelMessageInfo *info, int flag)
 {
 	GList *list = NULL;
 	EGwJunkEntry *entry;
-	gchar **email, *from;	
+	gchar **email = NULL, *from = NULL;	
 	CamelGroupwiseStore *gw_store= CAMEL_GROUPWISE_STORE(store);
 	CamelGroupwiseStorePrivate  *priv = gw_store->priv;
 	EGwConnection *cnc = cnc_lookup (priv);
@@ -471,7 +492,6 @@ update_junk_list (CamelStore *store, CamelMessageInfo *info, int flag)
 		goto error;
 
 	email = g_strsplit_set (from, "<>", -1);
-
 	if (!email[1])
 		goto error;
 
@@ -492,17 +512,17 @@ update_junk_list (CamelStore *store, CamelMessageInfo *info, int flag)
 			if (e_gw_connection_create_junk_entry (cnc, email[1], "email", "junk") == E_GW_CONNECTION_STATUS_OK);
 		g_list_foreach (list, (GFunc) free_node, NULL);
 	}
-error:	
+
+error:
 	g_free (from);
 	g_strfreev (email);
 }
 
 static void 
-move_to_mailbox (CamelFolder *folder, CamelMessageInfo *info)
+move_to_mailbox (CamelFolder *folder, CamelMessageInfo *info, CamelException *ex)
 {
 	CamelFolder *dest;
 	GPtrArray *uids;
-	CamelException *ex;
 	const char *uid = camel_message_info_uid (info);
 	
 	uids = g_ptr_array_new ();
@@ -512,24 +532,23 @@ move_to_mailbox (CamelFolder *folder, CamelMessageInfo *info)
 	if (dest)
 		groupwise_transfer_messages_to (folder, uids, dest, NULL, TRUE, ex);
 	else
-		g_warning ("No MailBox folder found");
+		g_warning ("No Mailbox folder found");
 
 	update_junk_list (folder->parent_store, info, REMOVE_JUNK_ENTRY);
 }
 
 static void 
-move_to_junk (CamelFolder *folder, CamelMessageInfo *info)
+move_to_junk (CamelFolder *folder, CamelMessageInfo *info, CamelException *ex)
 {
 	CamelFolder *dest;
 	CamelFolderInfo *fi;
 	GPtrArray *uids;
-	CamelException *ex;
 	const char *uid = camel_message_info_uid (info);
 	
 	uids = g_ptr_array_new ();
 	g_ptr_array_add (uids, (gpointer) uid);
  	
-	dest = camel_store_get_folder (folder->parent_store, JUNK_FOLDER, 0, NULL);
+	dest = camel_store_get_folder (folder->parent_store, JUNK_FOLDER, 0, ex);
 	if (dest)
 		groupwise_transfer_messages_to (folder, uids, dest, NULL, TRUE, ex);
 	else {
@@ -553,7 +572,7 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 	CamelGroupwiseStore *gw_store = CAMEL_GROUPWISE_STORE (folder->parent_store);
 	CamelGroupwiseFolder *gw_folder = CAMEL_GROUPWISE_FOLDER (folder);
 	CamelGroupwiseStorePrivate *priv = gw_store->priv;
-	CamelMessageInfo *info;
+	CamelMessageInfo *info = NULL;
 	CamelGroupwiseMessageInfo *gw_info;
 	GList *read_items = NULL, *deleted_items = NULL;
 	flags_diff_t diff;
@@ -568,20 +587,29 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 	container_id =  camel_groupwise_store_container_id_lookup (gw_store, folder->full_name) ;
 
 	CAMEL_SERVICE_LOCK (gw_store, connect_lock);
+	if (!camel_groupwise_store_connected (gw_store, ex)) {
+		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+		camel_exception_clear (ex);
+		return;
+	}
+	CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
 
 	count = camel_folder_summary_count (folder->summary);
+	CAMEL_GROUPWISE_FOLDER_LOCK (folder, cache_lock);
 	for (i=0 ; i <count ; i++) {
 		guint32 flags = 0;
 		info = camel_folder_summary_index (folder->summary, i);
 		gw_info = (CamelGroupwiseMessageInfo *) info;
 		
 		/**Junk Mail handling**/
+		if(!info)
+			continue;
 		flags = camel_message_info_flags (info);	
 
 		if ((flags & CAMEL_MESSAGE_JUNK) && !(flags & CAMEL_GW_MESSAGE_JUNK)) /*marked a message junk*/
-			move_to_junk (folder, info);
+			move_to_junk (folder, info, ex);
 		else if ((flags & CAMEL_MESSAGE_JUNK) && (flags & CAMEL_GW_MESSAGE_JUNK)) /*message was marked as junk, now unjunk*/ 
-			move_to_mailbox (folder, info);
+			move_to_mailbox (folder, info, ex);
 
 		if (gw_info && (gw_info->info.flags & CAMEL_MESSAGE_FOLDER_FLAGGED)) {
 			do_flags_diff (&diff, gw_info->server_flags, gw_info->info.flags);
@@ -595,22 +623,29 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 				if (diff.changed & CAMEL_MESSAGE_SEEN)
 					read_items = g_list_append (read_items, (char *)uid);
 				if (diff.changed & CAMEL_MESSAGE_DELETED) {
-					//deleted_items = g_list_append (deleted_items, (char *)uid);
+					/*deleted_items = g_list_append (deleted_items, (char *)uid);*/
+					CAMEL_SERVICE_LOCK (gw_store, connect_lock);
 					status = e_gw_connection_remove_item (cnc, container_id, uid);
 					if (status == E_GW_CONNECTION_STATUS_OK) {
+						CAMEL_GROUPWISE_FOLDER_LOCK (folder, cache_lock);
 						camel_folder_summary_remove (folder->summary, info);
 						camel_data_cache_remove(gw_folder->cache, "cache", uid, ex);
+						CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
 						i--; count--;
 					}
+					CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
 				}
 			}
 		}
 		camel_message_info_free (info);
 		info = NULL;
 	}
+	CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
 
 	if (read_items && g_list_length (read_items)) {
+		CAMEL_SERVICE_LOCK (gw_store, connect_lock);
 		e_gw_connection_mark_read (cnc, read_items);
+		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
 	}
 	if (deleted_items) {
 	/*	status = e_gw_connection_remove_items (cnc, container_id, deleted_items);
@@ -627,14 +662,14 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 	}
 
 	if (expunge) {
+		CAMEL_SERVICE_LOCK (gw_store, connect_lock);
 		status = e_gw_connection_purge_deleted_items (cnc);
 		if (status == E_GW_CONNECTION_STATUS_OK) {
 		}
+		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
 	}
 	
 	camel_folder_summary_save (folder->summary);
-
-	CAMEL_SERVICE_UNLOCK (folder->parent_store, connect_lock);
 }
 
 
@@ -683,6 +718,7 @@ camel_gw_folder_new(CamelStore *store, const char *folder_name, const char *fold
 
 	journal_file = g_strdup_printf ("%s/journal",folder_dir);
 	gw_folder->journal = camel_groupwise_journal_new (gw_folder, journal_file);
+	g_free (journal_file);
 	if (!gw_folder->journal) {
 		camel_object_unref (folder);
 		return NULL;
@@ -719,15 +755,43 @@ update_update (CamelSession *session, CamelSessionThreadMsg *msg)
 	struct _folder_update_msg *m = (struct _folder_update_msg *)msg;
 	EGwConnectionStatus status;
 	CamelException *ex = NULL;
-	
-	status = e_gw_connection_get_quick_messages (m->cnc, m->container_id, "id",
-			&(m->t_str), "All", NULL, NULL, -1, &(m->slist));
+
+	GList *item_list;
+	int cursor = 0;
+	const char *position = E_GW_CURSOR_POSITION_END;
+	gboolean done;
+
+	status = e_gw_connection_create_cursor (m->cnc, m->container_id, "id", NULL, &cursor);
 	if (status != E_GW_CONNECTION_STATUS_OK) {
-		//camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Authentication failed"));
 		g_warning ("ERROR update update\n");
-	} else {
-		gw_update_all_items (m->folder, m->slist, ex);
+		return ;
 	}
+
+	done = FALSE;
+	m->slist = NULL;
+
+	while (!done) {
+		item_list = NULL;
+		status = e_gw_connection_get_all_mail_uids (m->cnc, m->container_id, cursor, FALSE, READ_CURSOR_MAX_IDS, position, &item_list);
+		if (status != E_GW_CONNECTION_STATUS_OK) {
+			g_warning ("ERROR update update\n");
+			e_gw_connection_destroy_cursor (m->cnc, m->container_id, cursor);
+			return;
+		}
+		
+		if (!item_list  || g_list_length (item_list) == 0)
+			done = TRUE;
+		else {
+			for (;item_list; item_list = g_list_next (item_list)) {
+				m->slist = g_slist_prepend (m->slist, (char *)item_list->data);
+			}
+		}
+		g_list_free (item_list);
+		position = E_GW_CURSOR_POSITION_CURRENT;
+	}
+	e_gw_connection_destroy_cursor (m->cnc, m->container_id, cursor);
+
+	gw_update_all_items (m->folder, m->slist, ex);
 }
 
 static void
@@ -753,12 +817,29 @@ static void
 groupwise_refresh_info(CamelFolder *folder, CamelException *ex)
 {
 	CamelGroupwiseSummary *summary;
+	CamelStoreInfo *si;
 	summary = (CamelGroupwiseSummary *) folder->summary;
-	if (summary->time_string) {
-		g_print ("Refreshing folder:%s\n", summary->time_string);
+	/*
+	 * Checking for the summary->time_string here since the first the a
+	 * user views a folder, the read cursor is in progress, and the getQM
+	 * should not interfere with the process
+	 */
+	if (summary->time_string && (strlen (summary->time_string) > 0))  {
 		groupwise_refresh_folder(folder, ex);
-	} else 
-		g_print ("Not refreshing folder\n");
+		si = camel_store_summary_path ((CamelStoreSummary *)((CamelGroupwiseStore *)folder->parent_store)->summary, folder->full_name);
+		if (si) {
+			guint32 unread, total;
+			camel_object_get (folder, NULL, CAMEL_FOLDER_TOTAL, &total, CAMEL_FOLDER_UNREAD, &unread, NULL);
+			if (si->total != total || si->unread != unread) {
+				si->total = total;
+				si->unread = unread;
+				camel_store_summary_touch ((CamelStoreSummary *)((CamelGroupwiseStore *)folder->parent_store)->summary);
+			}
+			camel_store_summary_info_free ((CamelStoreSummary *)((CamelGroupwiseStore *)folder->parent_store)->summary, si);
+		}
+		camel_folder_summary_save (folder->summary);
+		camel_store_summary_save ((CamelStoreSummary *)((CamelGroupwiseStore *)folder->parent_store)->summary);
+	}
 }
 
 void
@@ -771,6 +852,7 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 	EGwConnection *cnc = cnc_lookup (priv);
 	CamelSession *session = ((CamelService *)folder->parent_store)->session;
 	gboolean is_proxy = folder->parent_store->flags & CAMEL_STORE_PROXY;
+	gboolean is_locked = TRUE;
 	int status;
 	GList *list = NULL;
 	GSList *slist = NULL, *sl;
@@ -786,7 +868,7 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 	container_id = g_strdup (camel_groupwise_store_container_id_lookup (gw_store, folder->full_name)) ;
 
 	if (!container_id) {
-		g_error ("\nERROR - Container id not present. Cannot refresh info for %s\n", folder->full_name);
+		g_warning ("\nERROR - Container id not present. Cannot refresh info for %s\n", folder->full_name);
 		return;
 	}
 
@@ -796,15 +878,11 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 	
 	CAMEL_SERVICE_LOCK (gw_store, connect_lock);
 
-	/* Its a good idea to sync up first and then refresh...
-	 * XXX:But how long does this take??? Bad if its slow
-	 */
-	groupwise_sync (folder, FALSE, ex);
-	/*Done....should refresh now.....*/
+	if (!camel_groupwise_store_connected (gw_store, ex)) 
+		goto end1;
 
 	if (!strcmp (folder->full_name, "Trash") || is_proxy) {
-		g_print ("get_items:%s\n", folder->full_name);
-		status = e_gw_connection_get_items (cnc, container_id, "recipient distribution created attachments subject status size", NULL, &list);
+		status = e_gw_connection_get_items (cnc, container_id, "peek recipient distribution delivered attachments subject status size", NULL, &list);
 		if (status != E_GW_CONNECTION_STATUS_OK) {
 			if (status ==E_GW_CONNECTION_STATUS_OTHER) {
 				g_warning ("Trash full....Empty Trash!!!!\n");
@@ -872,9 +950,13 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 		gw_store->current_folder = folder;
 	}
 
-	if (list)
+	if (list) {
 		gw_update_cache (folder, list, ex);
+	}
 	
+ 
+	CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+	is_locked = FALSE;
 
 	/*
 	 * The New and Modified items in the server have been updated in the summary. 
@@ -903,19 +985,19 @@ end2:
 	g_free (time_string);
 	g_free (container_id);
 end1:
-	CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+	if (is_locked)
+		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
 	return;
 }
 
 void
-gw_update_cache ( CamelFolder *folder, GList *item_list,CamelException *ex) 
+gw_update_cache ( CamelFolder *folder, GList *list, CamelException *ex) 
 {
 	CamelGroupwiseMessageInfo *mi = NULL;
 	CamelGroupwiseStore *gw_store = CAMEL_GROUPWISE_STORE (folder->parent_store);
 	CamelGroupwiseFolder *gw_folder = CAMEL_GROUPWISE_FOLDER(folder);
 	CamelGroupwiseStorePrivate *priv = gw_store->priv;
 	EGwConnection *cnc = cnc_lookup (priv);
-	GPtrArray *msg;
 	GSList *attach_list = NULL;
 	guint32 item_status, status_flags = 0;
 	CamelFolderChangeInfo *changes = NULL;
@@ -925,14 +1007,15 @@ gw_update_cache ( CamelFolder *folder, GList *item_list,CamelException *ex)
 	char *container_id = NULL;
 	gboolean is_junk = FALSE;
 	EGwItemStatus status;
+	GList *item_list = list;
 	int total_items = g_list_length (item_list), i=0;
 	
 	/*Assert lock*/
-	msg = g_ptr_array_new ();
 	changes = camel_folder_change_info_new ();
 	container_id = g_strdup (camel_groupwise_store_container_id_lookup (gw_store, folder->full_name));
 	if (!container_id) {
-		g_error ("\nERROR - Container id not present. Cannot refresh info\n");
+		g_warning ("\nERROR - Container id not present. Cannot refresh info\n");
+		camel_folder_change_info_free (changes);
 		return;
 	}
 	
@@ -945,7 +1028,7 @@ gw_update_cache ( CamelFolder *folder, GList *item_list,CamelException *ex)
 	for ( ; item_list != NULL ; item_list = g_list_next (item_list) ) {
 		EGwItem *temp_item = (EGwItem *)item_list->data;
 		EGwItem *item;
-		EGwItemType type;
+		EGwItemType type = E_GW_ITEM_TYPE_UNKNOWN;
 		EGwItemOrganizer *org;
 		char *temp_date = NULL;
 		const char *id;
@@ -973,23 +1056,24 @@ gw_update_cache ( CamelFolder *folder, GList *item_list,CamelException *ex)
 		}
 
 		/************************ First populate summary *************************/
+		mi = NULL;
 		mi = (CamelGroupwiseMessageInfo *)camel_folder_summary_uid (folder->summary, id);
-		if (mi) 
+		if (mi) {
 			exists = TRUE;
+		}
 
 		if (!exists) {
-			mi = camel_message_info_new (folder->summary); 
-			if (mi->info.content == NULL) {
-				mi->info.content = camel_folder_summary_content_info_new (folder->summary);
-				mi->info.content->type = camel_content_type_new ("multipart", "mixed");
-			}
-
 			type = e_gw_item_get_item_type (item);
 			if ((type == E_GW_ITEM_TYPE_CONTACT) || (type == E_GW_ITEM_TYPE_UNKNOWN)) {
 				exists = FALSE;
-				g_object_unref (item);
 				continue;
 			}
+
+			mi = (CamelGroupwiseMessageInfo *)camel_message_info_new (folder->summary); 
+			if (mi->info.content == NULL) {
+				mi->info.content = camel_folder_summary_content_info_new (folder->summary);
+				mi->info.content->type = camel_content_type_new ("multipart", "mixed");
+			}
 		}
 
 		/*all items in the Junk Mail folder should have this flag set*/
@@ -1027,6 +1111,8 @@ gw_update_cache ( CamelFolder *folder, GList *item_list,CamelException *ex)
 		org = e_gw_item_get_organizer (item); 
 		if (org) {
 			g_string_append_printf (str, "%s <%s>",org->display_name, org->email);
+			if (exists)
+				camel_pstring_free(mi->info.from);
 			mi->info.from = camel_pstring_strdup (str->str);
 		}
 		g_string_truncate (str, 0);
@@ -1043,11 +1129,15 @@ gw_update_cache ( CamelFolder *folder, GList *item_list,CamelException *ex)
 					i++;
 				}
 			}
+			if (exists)
+				camel_pstring_free(mi->info.to);
 			mi->info.to = camel_pstring_strdup (str->str);
 			g_string_truncate (str, 0);
 		}
 		
-		if (type == E_GW_ITEM_TYPE_APPOINTMENT) {
+		if (type == E_GW_ITEM_TYPE_APPOINTMENT
+		    || type ==  E_GW_ITEM_TYPE_NOTE 
+		    || type ==  E_GW_ITEM_TYPE_TASK ) {
 			temp_date = e_gw_item_get_start_date (item);
 			if (temp_date) {
 				time_t time = e_gw_connection_get_date_from_string (temp_date);
@@ -1055,7 +1145,7 @@ gw_update_cache ( CamelFolder *folder, GList *item_list,CamelException *ex)
 				mi->info.date_sent = mi->info.date_received = actual_time;
 			}
 		} else {
-			temp_date = e_gw_item_get_creation_date(item);
+			temp_date = e_gw_item_get_delivered_date(item);
 			if (temp_date) {
 				time_t time = e_gw_connection_get_date_from_string (temp_date);
 				time_t actual_time = camel_header_decode_date (ctime(&time), NULL);
@@ -1063,10 +1153,13 @@ gw_update_cache ( CamelFolder *folder, GList *item_list,CamelException *ex)
 			}
 		}
 
+		if (exists)
+			g_free(mi->info.uid);
 		mi->info.uid = g_strdup(e_gw_item_get_id(item));
 		if (!exists)
 			mi->info.size = e_gw_item_get_mail_size (item);	
-		mi->info.subject = camel_pstring_strdup(e_gw_item_get_subject(item));
+		if (!exists)
+			mi->info.subject = camel_pstring_strdup(e_gw_item_get_subject(item));
 		
 		if (exists) 
 			camel_folder_change_info_change_uid (changes, e_gw_item_get_id (item));
@@ -1077,10 +1170,14 @@ gw_update_cache ( CamelFolder *folder, GList *item_list,CamelException *ex)
 		}
 
 		/********************* Summary Stuff ends *************************/
-		g_ptr_array_add (msg, mi);
 		exists = FALSE;
+		if (!strcmp (folder->full_name, "Junk Mail")|| !strcmp (folder->full_name, "Sent Items"))
+			continue;
+
 		/******************** Begine Caching ************************/
 		mail_msg = groupwise_folder_item_to_msg (folder, item, ex);
+		if (mail_msg)
+			camel_medium_set_header (CAMEL_MEDIUM (mail_msg), "X-Evolution-Source", groupwise_base_url_lookup (priv));
 		/* add to cache if its a new message*/
 		if (!exists) {
 			CAMEL_GROUPWISE_FOLDER_LOCK (folder, cache_lock);
@@ -1094,7 +1191,6 @@ gw_update_cache ( CamelFolder *folder, GList *item_list,CamelException *ex)
 			CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
 		}
 		/******************** Caching stuff ends *************************/
-		g_object_unref (item);
 		i++;
 	}
 	camel_operation_end (NULL);
@@ -1103,15 +1199,13 @@ gw_update_cache ( CamelFolder *folder, GList *item_list,CamelException *ex)
 	camel_object_trigger_event (folder, "folder_changed", changes);
 
 	camel_folder_change_info_free (changes);
-	g_ptr_array_free (msg, TRUE);
 }
 
 void
-gw_update_summary ( CamelFolder *folder, GList *item_list,CamelException *ex) 
+gw_update_summary ( CamelFolder *folder, GList *list,CamelException *ex) 
 {
 	CamelGroupwiseMessageInfo *mi = NULL;
 	CamelGroupwiseStore *gw_store = CAMEL_GROUPWISE_STORE (folder->parent_store);
-	GPtrArray *msg;
 	GSList *attach_list = NULL;
 	guint32 item_status, status_flags = 0;
 	CamelFolderChangeInfo *changes = NULL;
@@ -1120,13 +1214,14 @@ gw_update_summary ( CamelFolder *folder, GList *item_list,CamelException *ex)
 	const char *priority = NULL;
 	char *container_id = NULL;
 	gboolean is_junk = FALSE;
+	GList *item_list = list;
 	
 	/*Assert lock???*/
-	msg = g_ptr_array_new ();
 	changes = camel_folder_change_info_new ();
 	container_id = g_strdup (camel_groupwise_store_container_id_lookup (gw_store, folder->full_name));
 	if (!container_id) {
-		g_error ("\nERROR - Container id not present. Cannot refresh info\n");
+		g_warning ("\nERROR - Container id not present. Cannot refresh info\n");
+		camel_folder_change_info_free (changes);
 		return;
 	}
 	
@@ -1134,9 +1229,9 @@ gw_update_summary ( CamelFolder *folder, GList *item_list,CamelException *ex)
 		is_junk = TRUE;
 	}
 
-	for ( ; item_list != NULL ; item_list = g_list_next (item_list) ) {
+	for (; item_list != NULL ; item_list = g_list_next (item_list) ) {
 		EGwItem *item = (EGwItem *)item_list->data;
-		EGwItemType type;
+		EGwItemType type = E_GW_ITEM_TYPE_UNKNOWN;
 		EGwItemOrganizer *org;
 		char *temp_date = NULL;
 		const char *id;
@@ -1146,22 +1241,23 @@ gw_update_summary ( CamelFolder *folder, GList *item_list,CamelException *ex)
 		id = e_gw_item_get_id (item);
 		
 		mi = (CamelGroupwiseMessageInfo *)camel_folder_summary_uid (folder->summary, id);
-		if (mi) 
+		if (mi) {
 			exists = TRUE;
+			camel_message_info_free (&mi->info);
+		}
 
 		if (!exists) {
+			type = e_gw_item_get_item_type (item);
+			if ((type == E_GW_ITEM_TYPE_CONTACT) || (type == E_GW_ITEM_TYPE_UNKNOWN)) {
+				exists = FALSE;
+				continue;
+			}
+
 			mi = camel_message_info_new (folder->summary); 
 			if (mi->info.content == NULL) {
 				mi->info.content = camel_folder_summary_content_info_new (folder->summary);
 				mi->info.content->type = camel_content_type_new ("multipart", "mixed");
 			}
-
-			type = e_gw_item_get_item_type (item);
-			if ((type == E_GW_ITEM_TYPE_CONTACT) || (type == E_GW_ITEM_TYPE_UNKNOWN)) {
-				exists = FALSE;
-				g_object_unref (item);
-				continue;
-			}
 		}
 
 		/*all items in the Junk Mail folder should have this flag set*/
@@ -1219,15 +1315,17 @@ gw_update_summary ( CamelFolder *folder, GList *item_list,CamelException *ex)
 			g_string_truncate (str, 0);
 		}
 		
-		if (type == E_GW_ITEM_TYPE_APPOINTMENT) {
+		if (type == E_GW_ITEM_TYPE_APPOINTMENT
+		    || type ==  E_GW_ITEM_TYPE_NOTE 
+		    || type ==  E_GW_ITEM_TYPE_TASK ) {
 			temp_date = e_gw_item_get_start_date (item);
 			if (temp_date) {
 				time_t time = e_gw_connection_get_date_from_string (temp_date);
 				time_t actual_time = camel_header_decode_date (ctime(&time), NULL);
 				mi->info.date_sent = mi->info.date_received = actual_time;
-			}
+			} 
 		} else {
-			temp_date = e_gw_item_get_creation_date(item);
+			temp_date = e_gw_item_get_delivered_date(item);
 			if (temp_date) {
 				time_t time = e_gw_connection_get_date_from_string (temp_date);
 				time_t actual_time = camel_header_decode_date (ctime(&time), NULL);
@@ -1248,7 +1346,6 @@ gw_update_summary ( CamelFolder *folder, GList *item_list,CamelException *ex)
 			camel_folder_change_info_recent_uid (changes, mi->info.uid);
 		}
 
-		g_ptr_array_add (msg, mi);
 		exists = FALSE;
 	}
 	g_free (container_id);
@@ -1256,7 +1353,6 @@ gw_update_summary ( CamelFolder *folder, GList *item_list,CamelException *ex)
 	camel_object_trigger_event (folder, "folder_changed", changes);
 
 	camel_folder_change_info_free (changes);
-	g_ptr_array_free (msg, TRUE);
 }
 
 static CamelMimeMessage *
@@ -1267,7 +1363,7 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 	CamelMimeMessage *msg = NULL;
 	CamelGroupwiseStore *gw_store = CAMEL_GROUPWISE_STORE(folder->parent_store);
 	CamelGroupwiseStorePrivate  *priv = gw_store->priv;
-	char *container_id = NULL;
+	const char *container_id = NULL;
 	GSList *attach_list = NULL;
 	EGwItemType type;
 	EGwConnectionStatus status;
@@ -1276,6 +1372,7 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 	int errno;
 	char *body = NULL;
 	const char *uid = NULL;
+	gboolean is_text_html = FALSE;
 	
 
 	uid = e_gw_item_get_id(item);
@@ -1301,10 +1398,11 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 				camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Could not get message"));
 				return NULL;
 			}
-			if (attachment && (len !=0) ) {
+			if (attachment && attachment[0] && (len !=0) ) {
 				if (!g_ascii_strcasecmp (attach->name, "TEXT.htm")) {
 					body = g_strdup (attachment);
 					g_free (attachment);
+					is_text_html = TRUE;
 				} 
 			}//if attachment and len
 		} // if Mime.822 or TEXT.htm
@@ -1352,6 +1450,7 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 	groupwise_populate_details_from_item (msg, item);
 	/*Now set attachments*/
 	if (attach_list) {
+		gboolean has_boundary = FALSE;
 		GSList *al;
 
 		for (al = attach_list ; al != NULL ; al = al->next) {
@@ -1367,10 +1466,9 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 			
 			if ( (attach->item_reference) && (!g_ascii_strcasecmp (attach->item_reference, "1")) ) {
 				CamelMimeMessage *temp_msg = NULL;
-				status = e_gw_connection_get_item (cnc, container_id, attach->id, "default distribution recipient message attachments subject notification created recipientStatus status", &temp_item);
+				status = e_gw_connection_get_item (cnc, container_id, attach->id, "default distribution recipient message attachments subject notification created recipientStatus status startDate", &temp_item);
 				if (status != E_GW_CONNECTION_STATUS_OK) {
 					g_warning ("Could not get attachment\n");
-					camel_object_unref (part);
 					continue;
 				}
 				temp_msg = groupwise_folder_item_to_msg(folder, temp_item, ex);
@@ -1395,21 +1493,39 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 				}
 				if (attachment && (len !=0) ) {
 					part = camel_mime_part_new ();
-					
-					if (!strcmp (attach->contentType, "application/pgp-signature")) {
-						camel_data_wrapper_set_mime_type(CAMEL_DATA_WRAPPER (multipart), "multipart/signed");
-						camel_content_type_set_param(CAMEL_DATA_WRAPPER (multipart)->mime_type, "protocol", attach->contentType);
-					} else if (!strcmp (attach->contentType, "application/pgp-encrypted")) {
-						camel_data_wrapper_set_mime_type(CAMEL_DATA_WRAPPER (multipart), "multipart/encrypted");
- 						camel_content_type_set_param(CAMEL_DATA_WRAPPER (multipart)->mime_type, "protocol", attach->contentType);
+					if (attach->contentType) {	
+						if (!strcmp (attach->contentType, "application/pgp-signature")) {
+							camel_mime_part_set_filename(part, g_strdup(attach->name));
+							camel_data_wrapper_set_mime_type(CAMEL_DATA_WRAPPER (multipart), "multipart/signed");
+							has_boundary = TRUE;
+							camel_content_type_set_param(CAMEL_DATA_WRAPPER (multipart)->mime_type, "protocol", attach->contentType);
+						} else if (!strcmp (attach->contentType, "application/pgp-encrypted")) {
+							camel_data_wrapper_set_mime_type(CAMEL_DATA_WRAPPER (multipart), "multipart/encrypted");
+							has_boundary = TRUE;
+							camel_content_type_set_param(CAMEL_DATA_WRAPPER (multipart)->mime_type, "protocol", attach->contentType);
+						} else if ( !strcmp (attach->name, "encrypted.asc") &&
+								!strcmp (attach->contentType, "application/octet-stream")) {
+							camel_mime_part_set_filename(part, g_strdup(attach->name));
+							camel_data_wrapper_set_mime_type(CAMEL_DATA_WRAPPER (multipart), "multipart/encrypted");
+							has_boundary = TRUE;
+							camel_content_type_set_param(CAMEL_DATA_WRAPPER (multipart)->mime_type, "protocol", "application/pgp-encrypted");
+						} else {
+							camel_mime_part_set_filename(part, g_strdup(attach->name));
+							camel_mime_part_set_content_id (part, attach->id);
+							if (!has_boundary) {
+								camel_data_wrapper_set_mime_type(CAMEL_DATA_WRAPPER (multipart), "multipart/digest");
+							}
+						}
 					} else {
+						camel_mime_part_set_filename(part, g_strdup(attach->name));
 						camel_mime_part_set_content_id (part, attach->id);
-						camel_data_wrapper_set_mime_type(CAMEL_DATA_WRAPPER (multipart), "multipart/digest");
+						if (!has_boundary)
+							camel_data_wrapper_set_mime_type(CAMEL_DATA_WRAPPER (multipart), "multipart/digest");
 					}
 
 					camel_multipart_set_boundary(multipart, NULL);
 
-					camel_mime_part_set_filename(part, g_strdup(attach->name));
+					//camel_mime_part_set_filename(part, g_strdup(attach->name));
 					camel_mime_part_set_content(part, attachment, len, attach->contentType);
 
 					camel_multipart_add_part (multipart, part);
@@ -1475,9 +1591,11 @@ gw_update_all_items ( CamelFolder *folder, GSList *item_list, CamelException *ex
 		temp = g_slist_find_custom (item_list, (const char *)item_ids->data, (GCompareFunc) string_cmp);
 		if (!temp) {
 			//g_print ("Deleting:%s\n", (const char *)item_ids->data);
+			CAMEL_GROUPWISE_FOLDER_LOCK (folder, cache_lock);
 			camel_folder_summary_remove_uid (folder->summary, (const char *)item_ids->data);
 			camel_data_cache_remove(gw_folder->cache, "cache", (const char *)item_ids->data, ex);
 			camel_folder_change_info_remove_uid(changes, (const char *)item_ids->data);
+			CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
 		} 
 	}
 	camel_object_trigger_event (folder, "folder_changed", changes);
@@ -1500,7 +1618,19 @@ groupwise_append_message (CamelFolder *folder, CamelMimeMessage *message,
 	EGwConnection *cnc = cnc_lookup (priv);
 	EGwItem *item;
 	char *id;
+	gboolean is_ok = FALSE;
 	
+	if (!strcmp (folder->name, RECEIVED))
+		is_ok = TRUE;
+	if(!strcmp (folder->name, SENT))
+		is_ok = TRUE;
+
+	if (!is_ok) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot append message to folder `%s': %s"),
+				folder->full_name, e_gw_connection_get_error_message (status));
+		return;
+	}
+
 	if (offline->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
 		camel_groupwise_journal_append ((CamelGroupwiseJournal *) ((CamelGroupwiseFolder *)folder)->journal, message, info, appended_uid, ex);
 		return;
@@ -1639,8 +1769,12 @@ groupwise_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 	
 	index = 0;
 	while (index < uids->len) {
-		status = e_gw_connection_move_item (cnc, (const char *)uids->pdata[index], 
-				                    dest_container_id, source_container_id);
+		if (delete_originals) 
+			status = e_gw_connection_move_item (cnc, (const char *)uids->pdata[index], 
+					dest_container_id, source_container_id);
+		else
+			status = e_gw_connection_move_item (cnc, (const char *)uids->pdata[index], 
+					dest_container_id, NULL);
 		if (status == E_GW_CONNECTION_STATUS_OK) {
 			const char *old_uid = (const char*)uids->pdata[index];
 			char **tmp;
@@ -1651,7 +1785,7 @@ groupwise_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 			if (!strcmp(source->full_name, JUNK_FOLDER))
 				camel_folder_set_message_flags (source, old_uid, CAMEL_GW_MESSAGE_JUNK|CAMEL_MESSAGE_JUNK|CAMEL_MESSAGE_JUNK_LEARN, 0);
 			else if (!strcmp(destination->full_name, JUNK_FOLDER))
-				camel_folder_set_message_flags (source, old_uid, CAMEL_MESSAGE_JUNK, CAMEL_GW_MESSAGE_JUNK);
+				camel_folder_set_message_flags (source, old_uid, CAMEL_MESSAGE_JUNK|CAMEL_GW_MESSAGE_JUNK, CAMEL_GW_MESSAGE_JUNK);
 
 			CamelGroupwiseMessageInfo *dest_info = (CamelGroupwiseMessageInfo *)camel_message_info_clone((CamelMessageInfo *)src_info);
 			tmp = g_strsplit (old_uid, ":", -1);
@@ -1664,9 +1798,10 @@ groupwise_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 			if (delete_originals) {
 				camel_folder_set_message_flags (source, (const char *)uids->pdata[index],
 						CAMEL_MESSAGE_DELETED, CAMEL_MESSAGE_DELETED);
+
 			}
 		} else {
-			g_print ("Warning!! Could not move item : %s\n", (char *)uids->pdata[index]);
+			g_warning ("Warning!! Could not move item : %s\n", (char *)uids->pdata[index]);
 		}
 		index ++;
 	}
@@ -1693,17 +1828,18 @@ groupwise_expunge (CamelFolder *folder, CamelException *ex)
 	int i, max;
 	gboolean delete = FALSE;
 	
-	CAMEL_SERVICE_LOCK (groupwise_store, connect_lock);
 	
 	cnc = cnc_lookup (priv);
 	if (!cnc)
 		return;
 
 	if (!strcmp (folder->full_name, "Trash")) {
+		CAMEL_SERVICE_LOCK (groupwise_store, connect_lock);
 		status = e_gw_connection_purge_deleted_items (cnc);
 		if (status == E_GW_CONNECTION_STATUS_OK) {
-			camel_folder_summary_clear (folder->summary);
-			camel_folder_summary_save (folder->summary);
+			camel_folder_freeze (folder);
+			groupwise_summary_clear (folder->summary, TRUE);
+			camel_folder_thaw (folder);
 		} else
 			g_warning ("Could not Empty Trash\n");
 		CAMEL_SERVICE_UNLOCK (groupwise_store, connect_lock);
@@ -1720,14 +1856,18 @@ groupwise_expunge (CamelFolder *folder, CamelException *ex)
 		ginfo = (CamelGroupwiseMessageInfo *) info;
 		if (ginfo && (ginfo->info.flags & CAMEL_MESSAGE_DELETED)) {
 			const char *uid = camel_message_info_uid (info);
+			CAMEL_SERVICE_LOCK (groupwise_store, connect_lock);
 			status = e_gw_connection_remove_item (cnc, container_id, uid);
 			if (status == E_GW_CONNECTION_STATUS_OK) {
+				CAMEL_GROUPWISE_FOLDER_LOCK (folder, cache_lock);
 				camel_folder_change_info_remove_uid (changes, (char *) uid);
 				camel_folder_summary_remove (folder->summary, info);
 				camel_data_cache_remove(gw_folder->cache, "cache", uid, ex);
+				CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
 				delete = TRUE;
 				i--;  max--;
 			}
+			CAMEL_SERVICE_UNLOCK (groupwise_store, connect_lock);
 		}
 		camel_message_info_free (info);
 	}
@@ -1735,7 +1875,6 @@ groupwise_expunge (CamelFolder *folder, CamelException *ex)
 	if (delete)
 		camel_object_trigger_event (CAMEL_OBJECT (folder), "folder_changed", changes);
 	
-	CAMEL_SERVICE_UNLOCK (groupwise_store, connect_lock);
 	
 	g_free (container_id);
 	camel_folder_change_info_free (changes);
@@ -1864,7 +2003,7 @@ convert_to_calendar (EGwItem *item, char **str, int *len)
 	GSList *attach_list = NULL;
 	GString *gstr = g_string_new (NULL);
 	int recur_key = 0;
-	char **tmp;
+	char **tmp = NULL;
 	const char *temp = NULL;
 
 	tmp = g_strsplit (e_gw_item_get_id (item), "@", -1);
@@ -1960,7 +2099,7 @@ convert_to_task (EGwItem *item, char **str, int *len)
 	EGwItemOrganizer *org = NULL;
 	GSList *recp_list = NULL;
 	GString *gstr = g_string_new (NULL);
-	char **tmp;
+	char **tmp = NULL;
 	const char *temp = NULL;
 	
 	tmp = g_strsplit (e_gw_item_get_id (item), "@", -1);
